/*
 * Copyright (C) 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.clearsilver.jsilver.exceptions;

import java.io.FileNotFoundException;

/**
 * Class to hold utilities related to exceptions used by JSilver code.
 */
public final class ExceptionUtil {
  private ExceptionUtil() {}

  /**
   * Determines if the given exception was caused by an exception equivalent to FileNotFound.
   */
  public static boolean isFileNotFoundException(Throwable th) {
    while (th != null) {
      if (th instanceof JSilverTemplateNotFoundException || th instanceof FileNotFoundException) {
        return true;
      }
      th = th.getCause();
    }
    return false;
  }
}
