/* This file was generated by SableCC (http://www.sablecc.org/). */

package com.google.clearsilver.jsilver.syntax.lexer;

@SuppressWarnings("serial")
public class LexerException extends Exception
{
    public LexerException(String message)
    {
        super(message);
    }
}
