/* This file was generated by SableCC (http://www.sablecc.org/). */

package com.google.clearsilver.jsilver.syntax.parser;

import java.util.ArrayList;

final class State
{
    int state;
    ArrayList nodes;

    State(@SuppressWarnings("hiding") int state, @SuppressWarnings("hiding") ArrayList nodes)
    {
        this.state = state;
        this.nodes = nodes;
    }
}
